// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// plugin.cpp - generic plugin operation

/**
*  Copyright (c) 2001, Will Day <willday@hpgx.net>, Tim Thelin <kndroc@hotmail.com>,
*  Pieter de Zwart <pdezwart@scu.edu>, David Mart
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*   In addition, as a special exception, <name of copyright
*   holder> gives permission to link the code of this program with
*   the FOO library (or with modified versions of FOO that use the
*   same license as FOO), and distribute linked combinations including
*   the two.  You must obey the GNU General Public License in all
*   respects for all of the code used other than FOO.  If you modify
*   this file, you may extend this exception to your version of the
*   file, but you are not obligated to do so.  If you do not wish to
*   do so, delete this exception statement from your version.
*
**/

/**
* This file is part of Log Daemon.
*
*   Log Daemon is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   Log Daemon is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Log Daemon; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/

#include <extdll.h>		// always
#include <sdk_util.h>
#include <meta_api.h>

#include "LogEvents.h"
#include "StandardParser.h"
#include "LogListener.h"
#include "log_logd.h"

#define CALL_LIST_SIZE ( HIGHEST_EVENT_CODE + 1 )

LogListener* callList[ CALL_LIST_SIZE ]; //used for a simple index table by log event.  Each
                                         //array cell is the head of a linked list
BOOL clean = FALSE;


//recursivly deletes a call list chain
void delList( LogListener* head )
{
    if( head )
    {
        delList( head->next );
        delete head->command;
        delete head;
    }
}


//called at the end of a map so that the next map things are ready to use (AM plugins are gonig to register each map)
void callListCleanup()
{
    for( int x = LOWEST_EVENT_CODE; x < CALL_LIST_SIZE; x++ )
    {
        if( callList[ x ] )
        {
            delList( callList[ x ] );
            callList[ x ] = 0;
        }
    }
}


//called by the parser when an event is recognized
void notifyListeners( int eventCode, char* params )
{
	LOG_DEBUG( 1, ( "Notifying listeners, Code: %i Paramters: %s", eventCode, params ) );
    LogListener* current = callList[ eventCode ];
    while( current ) //while we have actual structures in the list
    {
        LOG_DEBUG( 1, ("Server Command: %s %s", current->command, params)  );
        SERVER_COMMAND( UTIL_VarArgs("%s %s\n", current->command, params ) );
        current = current->next;
    }
}

//when something calls logd_register it calls this.
//this function adds a new listener to the listening chain
void addListener(void)
{
    int cmdNum = CMD_ARGC();
    //make sure there are enough args to make a listener out of
	if( cmdNum < 3 )
	{
	    if( cmdNum == 1 )
    	    LOG_MESSAGE( PLID, "ERROR: (logd_register) no parameters given" );
    	else
    	    LOG_MESSAGE( PLID, "ERROR: (logd_register) only 1 parameter given: %s", CMD_ARGV(1) );

	    return;
	}

	const char* strCode = CMD_ARGV(1);
    int len = strlen( strCode );
    int code = atoi( strCode );
    if( code < LOWEST_EVENT_CODE || code > HIGHEST_EVENT_CODE )
	{
	    LOG_MESSAGE( PLID, "ERROR: (logd_register) not a valid log code number: %s", strCode );
	    return;
	}

	const char* params = CMD_ARGS();
    if( params[ len ] != ' ' )
	{
	    LOG_MESSAGE( PLID, "ERROR: (logd_register) param format not correct, character %i is not a space", len );
	    return;
	}

    char* cmd = new char[ 64 ];
    strcpy( cmd, &params[ len + 1 ] );

    LogListener* current = callList[ code ];
    while( current )
    {
        if( strcmp( current->command, cmd ) == 0 )
        {
    	    LOG_MESSAGE( PLID, "ERROR: (logd_register) duplicate entry, ignoring this add attempt: %s", cmd );
    	    return;
    	}
    	current = current->next;
    }

    //create Listener and add it since it doesn't exist
    current = new LogListener;
    current->command = cmd;
    current->eventCode = code;
    current->next = callList[ code ];
    callList[ code ] = current;

    parseRoutineActive( code, TRUE );
}






void printListeners(void)
{
    LOG_CONSOLE( PLID, "\nLogD callback registration information:" );
    LOG_CONSOLE( PLID, "Event Name          Event Code   Function to call" );
    char* name;

    for( int x = 0; x < CALL_LIST_SIZE; x++ )
    {
        LogListener* current = callList[ x ];
        while( current )
        {
            name = transName( current->eventCode );
            LOG_CONSOLE( PLID, UTIL_VarArgs( "%-20s%-13i%s", name, current->eventCode, current->command) );

        	current = current->next;
        }
    }

}

void initializeListenerModule()
{
	LOG_DEBUG( 1, ( "Initializing listener module" ) );
    for( int z = 0; z < CALL_LIST_SIZE; z++ )
    {
        callList[ z ] = 0;      //all pointers are now null
    }
    REG_SVR_COMMAND("logd_reg", addListener);
    REG_SVR_COMMAND("logd_reginfo", printListeners);
    clean = TRUE;
}

void listenerActivate()
{
	LOG_DEBUG( 1, ( "Listener Activation" ) );
    if( !clean )
    {
        LOG_MESSAGE( PLID, "ERROR: callbacks never cleaned from last map, continuing but errors may get issued for multiple registrations." );
    }
    clean = FALSE;
}

void listenerDeactivate()
{
	LOG_DEBUG( 1, ("Listener Deactivation") );
    callListCleanup();
    clean = TRUE;
}


